import numpy as np

# Constants
DTOR = np.pi / 180.0
DAY_SECONDS = 86_400

EARTH_OCCULT = 20
SUN_OCCULT = 50
ANTISUN_OCCULT = 180
MOON_OCCULT = 20

# Figure out the
PANEL_CONSTRAINT = 45  # Above 90 degrees means panel is not illuminated
